@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2021 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2023 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -20,6 +20,10 @@ namespace BoCA
 	{
 		private:
 			Array<Bool>		 knownDrives;
+			Array<UnsignedInt32>	 knownDiscIDs;
+
+			Array<Array<Track> >	 runningConversions;
+			Array<Int>		 finishingConversions;
 		public:
 			static const String	&GetComponentSpecs();
 
@@ -30,6 +34,12 @@ namespace BoCA
 
 			Void			 OnApplicationAddTrack(const Track &);
 
+			Void			 OnStartConversion(Int, const Array<Track> &);
+			Void			 OnFinishConversion(Int);
+			Void			 OnFinishTrackConversion(Int, const Track &);
+
+			Void			 OnUpdateProtocol(const String &);
+
 			Void			 DoMenubarOverlay(GUI::Menu *);
 
 	};
