@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2016 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2021 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -20,12 +20,16 @@ namespace BoCA
 	class TagEdit : public CS::ExtensionComponent
 	{
 		private:
+			ConfigLayer		*configLayer;
+
 			LayerTags		*mainTabLayer;
 		public:
 			static const String	&GetComponentSpecs();
 
 						 TagEdit();
 						~TagEdit();
+
+			ConfigLayer		*GetConfigurationLayer();
 		callbacks:
 			Layer			*GetMainTabLayer();
 	};
