@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2020 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2022 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -90,42 +90,36 @@ String freac::CDDB::QueryStringFromMCDI(const MCDI &mcdi)
 
 UnsignedInt32 freac::CDDB::DiscIDFromOffsets(const String &offsets)
 {
-	Int	 numTocEntries = (Int64) Number::FromHexString(offsets);
+	const Array<String>	&values = offsets.Explode("+");
+
 	Int	 n = 0;
 
-	String	 remaining = offsets.Tail(offsets.Length() - offsets.Find("+") - 1);
-
-	for (Int i = 0; i < numTocEntries; i++)
+	for (Int i = 1; i < values.Length() - 1; i++)
 	{
-		Int	 offset = (Int64) Number::FromHexString(remaining);
+		Int	 offset = (Int64) Number::FromHexString(values.GetNth(i));
 
 		n += cddb_sum(offset / 75);
-
-		remaining = remaining.Tail(remaining.Length() - remaining.Find("+") - 1);
 	}
 
-	Int	 t = (Int64) Number::FromHexString(remaining) / 75 - (Int64) Number::FromHexString(offsets.Tail(offsets.Length() - offsets.Find("+") - 1)) / 75;
+	Int	 t = (Int64) Number::FromHexString(values.GetLast()) / 75 - (Int64) Number::FromHexString(values.GetNth(1)) / 75;
 
-	return ((n % 0xff) << 24 | t << 8 | numTocEntries);
+	return ((n % 0xff) << 24 | t << 8 | (values.Length() - 2));
 }
 
 String freac::CDDB::QueryStringFromOffsets(const String &offsets)
 {
-	Int	 numTocEntries = (Int64) Number::FromHexString(offsets);
+	const Array<String>	&values = offsets.Explode("+");
+
 	String	 str = String("cddb query ").Append(DiscIDToString(DiscIDFromOffsets(offsets)));
 
-	str.Append(" ").Append(String::FromInt(numTocEntries));
+	str.Append(" ").Append(String::FromInt(values.Length() - 2));
 
-	String	 remaining = offsets.Tail(offsets.Length() - offsets.Find("+") - 1);
-
-	for (Int i = 0; i < numTocEntries; i++)
+	for (Int i = 1; i < values.Length() - 1; i++)
 	{
-		str.Append(" ").Append(String::FromInt((Int64) Number::FromHexString(remaining)));
-
-		remaining = remaining.Tail(remaining.Length() - remaining.Find("+") - 1);
+		str.Append(" ").Append(String::FromInt((Int64) Number::FromHexString(values.GetNth(i))));
 	}
 
-	str.Append(" ").Append(String::FromInt((Int64) Number::FromHexString(remaining) / 75));
+	str.Append(" ").Append(String::FromInt((Int64) Number::FromHexString(values.GetLast()) / 75));
 
 	return str;
 }
@@ -448,34 +442,37 @@ Bool freac::CDDB::ParseCDDBRecord(const String &record, CDDBInfo &cddbInfo)
 
 /* Format a CDDB entry according to the rules
  * from the freedb how-to. Replace special
- * characters and split lines at 256 chars.
+ * characters and split lines at 256 chars
+ * minus two to account for line breaks.
  */
 String freac::CDDB::FormatCDDBEntry(const String &entry, const String &value)
 {
 	if (value == NIL) return String(entry).Append("=\n");
 
 	String	 result;
+	Int	 totalChars = value.Length();
+	Int	 lineChars  = 256 - entry.Length() - 1 - 2;
 
-	for (Int i = 0; i < value.Length(); )
+	for (Int i = 0; i < totalChars; )
 	{
 		String	 line = String(entry).Append("=");
 
-		for (Int c = 0; c < 254 - entry.Length() && i < value.Length(); c++, i++)
+		for (Int c = 0; c < lineChars && i < totalChars; c++, i++)
 		{
 			if (value[i] == '\n' || value[i] == '\t' || value[i] == '\\')
 			{
-				if (c >= 253 - entry.Length()) break;
+				if (c >= lineChars - 1) break;
 
 				if (value[i] == '\n') line.Append("\\n");
 				if (value[i] == '\t') line.Append("\\t");
 				if (value[i] == '\\') line.Append("\\\\");
 
 				c++;
+
+				continue;
 			}
-			else
-			{
-				line[line.Length()] = value[i];
-			}
+
+			line[line.Length()] = value[i];
 		}
 
 		result.Append(line).Append("\n");
