@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2021 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2023 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -25,6 +25,7 @@ const String	 freac::Config::CategorySettingsID				= "Settings";
 const String	 freac::Config::CategoryDialogsID				= "Dialogs";
 const String	 freac::Config::CategoryResourcesID				= "Resources";
 const String	 freac::Config::CategoryJoblistID				= "Joblist";
+const String	 freac::Config::CategorySplitterID				= "Splitter";
 const String	 freac::Config::CategoryRipperID				= "Ripper";
 const String	 freac::Config::CategoryPlaylistID				= "Playlist";
 const String	 freac::Config::CategoryTagsID					= "Tags";
@@ -130,6 +131,9 @@ const Bool	 freac::Config::SettingsFilenamesKeepTimeStampsDefault		= False;
 const String	 freac::Config::SettingsFilenamesAddSequentialNumbersID		= "FilenamesAddSequentialNumbers";
 const Bool	 freac::Config::SettingsFilenamesAddSequentialNumbersDefault	= False;
 
+const String	 freac::Config::SettingsFilenamesUseFallbackID			= "UseFilenameFallback";
+const Bool	 freac::Config::SettingsFilenamesUseFallbackDefault		= True;
+
 const String	 freac::Config::SettingsEncoderID				= "Encoder";
 const String	 freac::Config::SettingsEncoderDefault				= "lame-enc";
 
@@ -152,15 +156,16 @@ const String	 freac::Config::SettingsExcludeExtensionsID			= "ExcludeExtensions"
 const String	 freac::Config::SettingsExcludeExtensionsDefault		= String(".jpg|.jpeg|.gif|.png|.bmp|")		  // pictures
 										 .Append(".txt|.asc|.log|.nfo|.url|")		  // plain text
 										 .Append(".crc|.md5|.sha1|.sha256|.sha512|.sfv|") // checksums
-										 .Append(".m3u|.m3u8|.pls|.wpl|.xspf|")		  // playlists
+										 .Append(".m3u|.m3u8|.pls|.vclt|.wpl|.xspf|")	  // playlists
 										 .Append(".exe|.dll|")				  // executables
 										 .Append(".zip|.rar|.gz|.bz2|.xz|.7z|")		  // archives
-										 .Append(".ini|")				  // configuration
+										 .Append(".ini|.plist|")			  // configuration
 										 .Append(".htm|.html|.svg|.xml|")		  // HTML/SVG/XML
 										 .Append(".pdf|.ps|.xps|")			  // PDF etc.
 										 .Append(".doc|.docx|.odt|")			  // text documents
 										 .Append(".ppt|.pptx|.odp|")			  // presentations
 										 .Append(".xls|.xlsx|.ods|.odc|")		  // spreadsheets
+										 .Append(".itdb|.musicdb|")			  // iTunes database
 										 .Append(".torrent|thumbs.db|.DS_Store");	  // other files
 
 const String	 freac::Config::SettingsLastSelectedOpenDirID			= "LastSelectedOpenDir";
@@ -250,6 +255,48 @@ const String	 freac::Config::JoblistFieldsDefault				= "<artist>,<title>,<track>
 const String	 freac::Config::JoblistFieldSizesID				= "FieldSizes";
 const String	 freac::Config::JoblistFieldSizesDefault			= "120,*,50,80,100";
 
+/* Category Splitter
+ */
+const String	 freac::Config::SplitterSplitModeID				= "SplitMode";
+const Int	 freac::Config::SplitterSplitModeDefault			= 0;
+
+const String	 freac::Config::SplitterSplitAtTracksID				= "SplitAtTracks";
+const Bool	 freac::Config::SplitterSplitAtTracksDefault			= True;
+
+const String	 freac::Config::SplitterPartDurationModeID			= "PartDurationMode";
+const Int	 freac::Config::SplitterPartDurationModeDefault			= 1;
+
+const String	 freac::Config::SplitterPartDurationID				= "PartDuration";
+const Int	 freac::Config::SplitterPartDurationDefault			= 15;
+
+const String	 freac::Config::SplitterPartDurationUnitID			= "PartDurationUnit";
+const Int	 freac::Config::SplitterPartDurationUnitDefault			= 1;
+
+const String	 freac::Config::SplitterNumberOfPartsID				= "NumberOfParts";
+const Int	 freac::Config::SplitterNumberOfPartsDefault			= 4;
+
+const String	 freac::Config::SplitterMetadataStringID			= "MetadataString";
+const String	 freac::Config::SplitterMetadataStringDefault			= "<albumartist>,<album>";
+
+const String	 freac::Config::SplitterFileNamePatternID			= "FileNamePattern";
+const String	 freac::Config::SplitterFileNamePatternDefault			= "<albumartist> - <album>";
+
+const String	 freac::Config::SplitterRepeatEndID				= "RepeatEnd";
+const Bool	 freac::Config::SplitterRepeatEndDefault			= False;
+
+const String	 freac::Config::SplitterRepeatSecondsID				= "RepeatSeconds";
+const Int	 freac::Config::SplitterRepeatSecondsDefault			= 10;
+
+const String	 freac::Config::SplitterWindowPosXID				= "WindowPosX";
+
+const String	 freac::Config::SplitterWindowPosYID				= "WindowPosY";
+
+const String	 freac::Config::SplitterWindowSizeXID				= "WindowSizeX";
+const Int	 freac::Config::SplitterWindowSizeXDefault			= 750;
+
+const String	 freac::Config::SplitterWindowSizeYID				= "WindowSizeY";
+const Int	 freac::Config::SplitterWindowSizeYDefault			= 500;
+
 /* Category Ripper
  */
 const String	 freac::Config::RipperActiveDriveID				= "ActiveDrive";
@@ -289,6 +336,12 @@ const String	 freac::Config::PlaylistFilenamePatternDefault			= String("<albumar
 const String	 freac::Config::PlaylistCreateSingleFileID			= "SinglePlaylistFile";
 const Bool	 freac::Config::PlaylistCreateSingleFileDefault			= False;
 
+const String	 freac::Config::PlaylistFilenameID				= "PlaylistFilename";
+const String	 freac::Config::PlaylistFilenameDefault				= NIL;
+
+const String	 freac::Config::PlaylistCueSheetID				= "CueSheetFilename";
+const String	 freac::Config::PlaylistCueSheetDefault				= NIL;
+
 /* Category Tags
  */
 const String	 freac::Config::TagsCoverArtReadFromTagsID			= "CoverArtReadFromTags";
@@ -297,8 +350,14 @@ const Bool	 freac::Config::TagsCoverArtReadFromTagsDefault			= True;
 const String	 freac::Config::TagsCoverArtReadFromFilesID			= "CoverArtReadFromFiles";
 const Bool	 freac::Config::TagsCoverArtReadFromFilesDefault		= True;
 
+const String	 freac::Config::TagsCoverArtUsePatternsID			= "CoverArtRestrictFilenames";
+const Bool	 freac::Config::TagsCoverArtUsePatternsDefault			= True;
+
+const String	 freac::Config::TagsCoverArtPatternsID				= "CoverArtFilenames";
+const String	 freac::Config::TagsCoverArtPatternsDefault			= "folder;*cover*;*albumart*;*front*;*back*;*inside*;*cd*;*disc*;*booklet*";
+
 const String	 freac::Config::TagsCoverArtMaxFileSizeID			= "CoverArtMaxFileSize";
-const Int	 freac::Config::TagsCoverArtMaxFileSizeDefault			= 100;
+const Int	 freac::Config::TagsCoverArtMaxFileSizeDefault			= 250;
 
 const String	 freac::Config::TagsCoverArtWriteToTagsID			= "CoverArtWriteToTags";
 const Bool	 freac::Config::TagsCoverArtWriteToTagsDefault			= True;
@@ -330,6 +389,9 @@ const Bool	 freac::Config::TagsWriteMCDIDefault				= True;
 const String	 freac::Config::TagsPreserveReplayGainID			= "PreserveReplayGain";
 const Bool	 freac::Config::TagsPreserveReplayGainDefault			= True;
 
+const String	 freac::Config::TagsExtractFromFileNamesID			= "ExtractFromFileNames";
+const Bool	 freac::Config::TagsExtractFromFileNamesDefault			= True;
+
 const String	 freac::Config::TagsReplaceExistingCommentsID			= "ReplaceExistingComments";
 const Bool	 freac::Config::TagsReplaceExistingCommentsDefault		= False;
 
