@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2021 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2022 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -1766,7 +1766,7 @@ Void freac::LayerJoblist::OnBrowseForFolder()
 	DirSelection	 dialog;
 
 	dialog.SetParentWindow(mainWnd);
-	dialog.SetCaption(String("\n").Append(i18n->AddColon(i18n->TranslateString("Select the folder in which the encoded files will be placed"))));
+	dialog.SetCaption(i18n->AddColon(i18n->TranslateString("Select the folder in which the encoded files will be placed")));
 	dialog.SetDirName(BoCA::Utilities::GetAbsolutePathName(config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderOutputDirectoryID, Config::SettingsEncoderOutputDirectoryDefault)));
 
 	if (dialog.ShowDialog() == Success())
@@ -1788,7 +1788,7 @@ Void freac::LayerJoblist::OnSelectFolder()
 	 */
 	Directory	 outputDirectory(BoCA::Utilities::GetAbsolutePathName(edb_outdir->GetText()));
 
-	if (Directory::SetActiveDirectory(outputDirectory) != Success())
+	if (!outputDirectory.Exists())
 	{
 		Int	 selection = QuickMessage(i18n->TranslateString("The output folder does not exist! Do you want to create it?"), i18n->TranslateString("Error"), Message::Buttons::YesNoCancel, Message::Icon::Question);
 
@@ -1796,8 +1796,6 @@ Void freac::LayerJoblist::OnSelectFolder()
 		else if (selection == Message::Button::Cancel)	{ UpdateOutputDir(); return; }
 	}
 
-	Directory::SetActiveDirectory(GUI::Application::GetApplicationDirectory());
-
 	/* Save output directory and list of last used folders.
 	 */
 	Int	 outputDirNumber = 5;
