@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2020 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2023 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -284,6 +284,9 @@ Error freac::JobConvert::Perform()
 	Bool	 createPlaylist		= configuration->GetIntValue(Config::CategoryPlaylistID, Config::PlaylistCreatePlaylistID, Config::PlaylistCreatePlaylistDefault);
 	Bool	 createCueSheet		= configuration->GetIntValue(Config::CategoryPlaylistID, Config::PlaylistCreateCueSheetID, Config::PlaylistCreateCueSheetDefault);
 
+	String	 playlistFilename	= configuration->GetStringValue(Config::CategoryPlaylistID, Config::PlaylistFilenameID, Config::PlaylistFilenameDefault);
+	String	 cueSheetFilename	= configuration->GetStringValue(Config::CategoryPlaylistID, Config::PlaylistCueSheetID, Config::PlaylistCueSheetDefault);
+
 	Bool	 singlePlaylistFile	= configuration->GetIntValue(Config::CategoryPlaylistID, Config::PlaylistCreateSingleFileID, Config::PlaylistCreateSingleFileDefault);
 
 	/* Do not verify output if meh! encoder selected.
@@ -1112,6 +1115,12 @@ Error freac::JobConvert::Perform()
 
 			/* Delete processor and encoder.
 			 */
+			singleFileProcessor->Destroy();
+			singleFileEncoder->Destroy();
+
+			if (singleFileProcessor->GetErrorState()) BoCA::Utilities::ErrorMessage(singleFileProcessor->GetErrorString());
+			if (singleFileEncoder->GetErrorState())	  BoCA::Utilities::ErrorMessage(singleFileEncoder->GetErrorString());
+
 			delete singleFileProcessor;
 			delete singleFileEncoder;
 		}
@@ -1266,7 +1275,7 @@ Error freac::JobConvert::Perform()
 				log->Write(String("    Writing playlist: ").Append(Utilities::FormatFileNameForLogging(playlistFileNames.GetNth(i).Append(".").Append(playlistExtension), playlistBasePaths.GetNth(i))));
 
 				playlist->SetTrackList(*playlistTrackLists.GetNth(i));
-				playlist->WritePlaylist(String(playlistFileNames.GetNth(i)).Append(".").Append(playlistExtension));
+				playlist->WritePlaylist(playlistFilename == NIL ? String(playlistFileNames.GetNth(i)).Append(".").Append(playlistExtension) : playlistFilename);
 
 				boca.DeleteComponent(playlist);
 			}
@@ -1280,7 +1289,7 @@ Error freac::JobConvert::Perform()
 				log->Write(String("    Writing cue sheet: ").Append(Utilities::FormatFileNameForLogging(playlistFileNames.GetNth(i).Append(".cue"), playlistBasePaths.GetNth(i))));
 
 				cuesheet->SetTrackList(*cuesheetTrackLists.GetNth(i));
-				cuesheet->WritePlaylist(playlistFileNames.GetNth(i).Append(".cue"));
+				cuesheet->WritePlaylist(cueSheetFilename == NIL ? playlistFileNames.GetNth(i).Append(".cue") : cueSheetFilename);
 
 				boca.DeleteComponent(cuesheet);
 			}
@@ -1351,10 +1360,10 @@ Error freac::JobConvert::Perform()
 
 	delete progress;
 
-	/* Notify components and write log.
+	/* Finish log writing and notify components.
 	 */
-	if (abort) { engine->onCancelConversion.Emit(conversionID); log->Write("Conversion process cancelled.", MessageTypeWarning); }
-	else	   { engine->onFinishConversion.Emit(conversionID); log->Write("Conversion process finished."); }
+	if (abort) { log->Write("Conversion process cancelled.", MessageTypeWarning); engine->onCancelConversion.Emit(conversionID); }
+	else	   { log->Write("Conversion process finished.");		      engine->onFinishConversion.Emit(conversionID); }
 
 	/* Set progress to 100%.
 	 */
