@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2019 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2021 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -46,7 +46,7 @@ freac::JobManager::JobManager()
 
 freac::JobManager::~JobManager()
 {
-	exitThread = True;
+	Threads::Access::Set(exitThread, True);
 
 	/* Wait for manager thread to exit.
 	 */
@@ -55,31 +55,53 @@ freac::JobManager::~JobManager()
 	/* Disconnect slots.
 	 */
 	Job::onFinishJob.Disconnect(&JobManager::OnFinishJob, this);
+
+	/* Delete remaining job objects.
+	 */
+	const Array<Job *>	&all = Job::GetAllJobs();
+
+	foreachreverse (Job *job, all) Object::DeleteObject(job);
 }
 
 Void freac::JobManager::ManagerThread()
 {
 	BoCA::Config	*config = BoCA::Config::Get();
 
-	while (!exitThread)
+	while (!Threads::Access::Value(exitThread))
 	{
 		const Array<Job *>	&scheduled = Job::GetScheduledJobs();
 		const Array<Job *>	&planned   = Job::GetPlannedJobs();
 
+		/* Check scheduled jobs;
+		 */
+		scheduled.LockForRead();
+
 		if (scheduled.Length() > 0)
 		{
 			AutoRelease	 autoRelease;
 
 			foreachreverse (Job *job, scheduled)
 			{
+				scheduled.Unlock();
+
 				/* Call the job's precheck method.
 				 */
 				if (job->RunPrecheck() != Success()) Object::DeleteObject(job);
+
+				scheduled.LockForRead();
 			}
 
+			scheduled.Unlock();
+
 			continue;
 		}
 
+		scheduled.Unlock();
+
+		/* Check planned jobs.
+		 */
+		planned.LockForRead();
+
 		if (planned.Length() > 0)
 		{
 			AutoRelease	 autoRelease;
@@ -92,12 +114,29 @@ Void freac::JobManager::ManagerThread()
 
 				NonBlocking0<>(&Job::Run, job).Call();
 
-				while (planned.Get(job->GetHandle()) == job) S::System::System::Sleep(0);
+				/* Wait until job is removed from planned list.
+				 */
+				while (True)
+				{
+					if (planned.Get(job->GetHandle()) == NIL) break;
+
+					planned.Unlock();
+
+					S::System::System::Sleep(0);
+
+					planned.LockForRead();
+				}
 
 				break;
 			}
 		}
 
+		planned.Unlock();
+
+		/* Check finished jobs.
+		 */
+		finished.LockForRead();
+
 		if (finished.Length() > 0)
 		{
 			AutoRelease	 autoRelease;
@@ -105,9 +144,13 @@ Void freac::JobManager::ManagerThread()
 			Bool	 enableConsole = config->GetIntValue(Config::CategorySettingsID, Config::SettingsEnableConsoleID, Config::SettingsEnableConsoleDefault);
 			Bool	 displayErrors = config->GetIntValue(Config::CategorySettingsID, Config::SettingsDisplayErrorsID, Config::SettingsDisplayErrorsDefault);
 
+			finished.Unlock();
+			finished.LockForWrite();
+
 			foreachreverse (Job *job, finished)
 			{
 				finished.Remove(job->GetHandle());
+				finished.Unlock();
 
 				/* Check for errors and display error list if any.
 				 */
@@ -120,6 +163,8 @@ Void freac::JobManager::ManagerThread()
 						dialog.ShowDialog();
 					}
 
+					finished.LockForWrite();
+
 					continue;
 				}
 
@@ -127,13 +172,21 @@ Void freac::JobManager::ManagerThread()
 				 */
 				if (job->GetWarnings().Length() > 0 && !enableConsole)
 				{
+					finished.LockForWrite();
+
 					continue;
 				}
 
 				Object::DeleteObject(job);
+
+				finished.LockForWrite();
 			}
 		}
 
+		finished.Unlock();
+
+		/* Wait 100ms before checking jobs again.
+		 */
 		S::System::System::Sleep(100);
 	}
 }
