@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2021 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2023 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -31,6 +31,8 @@
 #include <dialogs/adddirectory.h>
 #include <dialogs/addpattern.h>
 
+#include <dialogs/splitter.h>
+
 #include <dialogs/error.h>
 
 #include <cddb/cddbremote.h>
@@ -253,10 +255,16 @@ freac::freacGUI::freacGUI()
 	if (config->GetIntValue(Config::CategorySettingsID, Config::SettingsShowTipsID, Config::SettingsShowTipsDefault)) mainWnd->onShow.Connect(&freacGUI::ShowTipOfTheDay, this);
 
 	mainWnd->doClose.Connect(&freacGUI::ExitProc, this);
-	mainWnd->SetMinimumSize(Size(600, 400 + (config->GetIntValue(Config::CategorySettingsID, Config::SettingsShowTitleInfoID, Config::SettingsShowTitleInfoDefault) ? 68 : 0)));
+	mainWnd->SetMinimumSize(Size(680, 460 + (config->GetIntValue(Config::CategorySettingsID, Config::SettingsShowTitleInfoID, Config::SettingsShowTitleInfoDefault) ? 68 : 0)));
 
 	if (config->GetIntValue(Config::CategorySettingsID, Config::SettingsWindowMaximizedID, Config::SettingsWindowMaximizedDefault)) mainWnd->Maximize();
 
+	/* Listen to joblist changes.
+	 */
+	BoCA::JobList::Get()->onApplicationAddTrack.Connect(&freacGUI::OnAddTrack, this);
+	BoCA::JobList::Get()->onApplicationRemoveTrack.Connect(&freacGUI::OnRemoveTrack, this);
+	BoCA::JobList::Get()->onApplicationRemoveAllTracks.Connect(&freacGUI::OnRemoveAllTracks, this);
+
 	/* Parse arguments and add files to joblist.
 	 */
 	ParseArguments(GetArguments());
@@ -268,6 +276,12 @@ freac::freacGUI::freacGUI()
 
 freac::freacGUI::~freacGUI()
 {
+	/* Disconnect joblist change slots.
+	 */
+	BoCA::JobList::Get()->onApplicationAddTrack.Disconnect(&freacGUI::OnAddTrack, this);
+	BoCA::JobList::Get()->onApplicationRemoveTrack.Disconnect(&freacGUI::OnRemoveTrack, this);
+	BoCA::JobList::Get()->onApplicationRemoveAllTracks.Disconnect(&freacGUI::OnRemoveAllTracks, this);
+
 	/* Free Notification instance.
 	 */
 	Notification::Free();
@@ -673,6 +687,39 @@ Void freac::freacGUI::OnChangeConfiguration()
 	surface->EndPaint();
 }
 
+Void freac::freacGUI::OnUpdateTracks(Bool removed, Bool removedAll)
+{
+	if (joblist->GetNOfTracks() != 1 && !removedAll) return;
+
+	BoCA::I18n	*i18n = BoCA::I18n::Get();
+
+	for (Int i = 0; i < menu_encode->Length(); i++)
+	{
+		MenuEntry	*entry = menu_encode->GetNthEntry(i);
+
+		if (entry->GetText() == i18n->AddEllipsis(i18n->TranslateString("Output splitting tool", "Dialogs::Splitter")))
+		{
+			if (removed) entry->Deactivate();
+			else	     entry->Activate();
+
+			break;
+		}
+	}
+
+	for (Int i = 0; i < mainWnd_iconbar->Length(); i++)
+	{
+		MenuEntry	*entry = mainWnd_iconbar->GetNthEntry(i);
+
+		if (entry->GetTooltipText() == i18n->TranslateString("Open the output splitting tool", "Toolbar"))
+		{
+			if (removed) entry->Deactivate();
+			else	     entry->Activate();
+
+			break;
+		}
+	}
+}
+
 Void freac::freacGUI::OnDriveChange()
 {
 	/* Update menus to reflect drive change.
@@ -836,12 +883,19 @@ Void freac::freacGUI::QueryCDDB()
 
 				if (trackNumber == -1) continue;
 
-				info.artist	 = cdInfo.GetTrackArtist(trackNumber);
-				info.title	 = cdInfo.trackTitles.GetNth(trackNumber - 1);
-				info.album	 = cdInfo.dTitle;
-				info.genre	 = cdInfo.dGenre;
-				info.year	 = cdInfo.dYear;
-				info.track	 = trackNumber;
+				info.artist = cdInfo.GetTrackArtist(trackNumber);
+
+				if (cdInfo.trackTitles.GetNth(trackNumber - 1) != NIL) info.title = cdInfo.trackTitles.GetNth(trackNumber - 1);
+
+				info.album = cdInfo.dTitle;
+				info.genre = cdInfo.dGenre;
+				info.year  = cdInfo.dYear;
+				info.track = trackNumber;
+
+				/* Set album artist.
+				 */
+				if (cdInfo.dArtist == CDDBInfo::VariousArtistsID) info.SetOtherInfo(BoCA::INFO_ALBUMARTIST, i18n->TranslateString("Various artists", "CDDB::Submit"));
+				else						  info.SetOtherInfo(BoCA::INFO_ALBUMARTIST, cdInfo.dArtist);
 
 				track.SetInfo(info);
 				track.SetOriginalInfo(info);
@@ -1026,8 +1080,8 @@ Void freac::freacGUI::FillMenus()
 	{
 		menu_file->AddEntry();
 
-		menu_file->AddEntry(i18n->AddEllipsis(i18n->TranslateString("Load joblist")))->onAction.Connect(&JobList::LoadList, joblist);
-		menu_file->AddEntry(i18n->AddEllipsis(i18n->TranslateString("Save joblist")))->onAction.Connect(&JobList::SaveList, joblist);
+		menu_file->AddEntry(i18n->AddEllipsis(i18n->TranslateString("Load joblist")))->onAction.Connect(&JobList::OpenJobList, joblist);
+		menu_file->AddEntry(i18n->AddEllipsis(i18n->TranslateString("Save joblist")))->onAction.Connect(&JobList::SaveJobList, joblist);
 	}
 
 	menu_file->AddEntry();
@@ -1163,7 +1217,9 @@ Void freac::freacGUI::FillMenus()
 	entry = menu_processing->AddEntry(i18n->TranslateString("Enable signal processing"), (Bool *) &config->GetPersistentIntValue(Config::CategoryProcessingID, Config::ProcessingEnableProcessingID, Config::ProcessingEnableProcessingDefault));
 	entry->onAction.Connect(&freacGUI::ToggleSignalProcessing, this);
 
-	menu_processing->AddEntry(i18n->TranslateString("Enable processing during playback"), (Bool *) &config->GetPersistentIntValue(Config::CategoryProcessingID, Config::ProcessingProcessPlaybackID, Config::ProcessingProcessPlaybackDefault));
+	entry = menu_processing->AddEntry(i18n->TranslateString("Enable processing during playback"), (Bool *) &config->GetPersistentIntValue(Config::CategoryProcessingID, Config::ProcessingProcessPlaybackID, Config::ProcessingProcessPlaybackDefault));
+
+	if (!config->GetIntValue(Config::CategoryProcessingID, Config::ProcessingEnableProcessingID, Config::ProcessingEnableProcessingDefault)) entry->Deactivate();
 
 	const Array<String>	&components = config->GetStringValue(Config::CategoryProcessingID, Config::ProcessingComponentsID, Config::ProcessingComponentsDefault).Explode(",");
 
@@ -1184,8 +1240,6 @@ Void freac::freacGUI::FillMenus()
 		menu_processing->AddEntry(i18n->TranslateString("Selected signal processors"), menu_processors);
 	}
 
-	ToggleSignalProcessing();
-
 	/* Fill conversion menu.
 	 */
 	i18n->SetContext("Menu::Encode");
@@ -1241,6 +1295,12 @@ Void freac::freacGUI::FillMenus()
 		menu_encode->AddEntry(i18n->AddEllipsis(i18n->TranslateString("Start encoding with")), menu_encoders);
 	}
 
+	menu_encode->AddEntry();
+	entry = menu_encode->AddEntry(i18n->AddEllipsis(i18n->TranslateString("Output splitting tool", "Dialogs::Splitter")));
+	entry->onAction.Connect(&freacGUI::ShowSplitterTool, this);
+
+	if (joblist->GetNOfTracks() == 0) entry->Deactivate();
+
 	menu_encoder_options->AddEntry(i18n->TranslateString("Encode to a single file"), (Bool *) &config->GetPersistentIntValue(Config::CategorySettingsID, Config::SettingsEncodeToSingleFileID, Config::SettingsEncodeToSingleFileDefault));
 
 	menu_encoder_options->AddEntry();
@@ -1359,6 +1419,14 @@ Void freac::freacGUI::FillMenus()
 
 	mainWnd_iconbar->AddEntry();
 
+	entry = mainWnd_iconbar->AddEntry(ImageLoader::Load(String(currentConfig->resourcesPath).Append("icons/tools/splitter.png")));
+	entry->onAction.Connect(&freacGUI::ShowSplitterTool, this);
+	entry->SetTooltipText(i18n->TranslateString("Open the output splitting tool"));
+
+	if (joblist->GetNOfTracks() == 0) entry->Deactivate();
+
+	mainWnd_iconbar->AddEntry();
+
 	for (Int i = 0; i < 3; i++)
 	{
 		switch (i18n->IsActiveLanguageRightToLeft() ? 2 - i : i)
@@ -1587,6 +1655,12 @@ Void freac::freacGUI::ToggleSignalProcessing()
 
 	if (config->GetIntValue(Config::CategoryProcessingID, Config::ProcessingEnableProcessingID, Config::ProcessingEnableProcessingDefault)) menu_processing->GetNthEntry(1)->Activate();
 	else																	menu_processing->GetNthEntry(1)->Deactivate();
+
+	/* Notify about changed setting state (main window
+	 * visibility is set to False in OnChangeConfiguration
+	 * so we know we were not called from there).
+	 */
+	if (mainWnd->IsVisible()) OnChangeConfiguration();
 }
 
 Void freac::freacGUI::ToggleUseInputDirectory()
@@ -1700,6 +1774,21 @@ Void freac::freacGUI::ShowTipOfTheDay()
 	config->SetIntValue(Config::CategorySettingsID, Config::SettingsNextTipID, dialog.GetOffset());
 }
 
+Void freac::freacGUI::ShowSplitterTool()
+{
+	/* Build track list according to current order.
+	 */
+	Array<Track>	 trackList;
+
+	for (Int i = 0; i < joblist->GetNOfTracks(); i++) trackList.Add(joblist->GetNthTrack(i));
+
+	/* Open splitter dialog.
+	 */
+	DialogSplitter	 dialog(trackList);
+
+	dialog.ShowDialog();
+}
+
 Void freac::freacGUI::ReportIssue()
 {
 	S::System::System::OpenURL(issueTracker);
