@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2021 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2022 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -25,6 +25,31 @@ using namespace BoCA::AS;
 using namespace smooth::GUI::Dialogs;
 using namespace smooth::IO;
 
+static const String	 kDot		     = ".";
+static const String	 kColon		     = ":";
+
+static const String	 kArtistPattern	     = "<artist>";
+static const String	 kTitlePattern	     = "<title>";
+static const String	 kAlbumPattern	     = "<album>";
+static const String	 kGenrePattern	     = "<genre>";
+static const String	 kYearPattern	     = "<year>";
+static const String	 kDiscIDPattern	     = "<discid>";
+static const String	 kFileNamePattern    = "<filename>";
+static const String	 kFileTypePattern    = "<filetype>";
+static const String	 kDirectoryPattern   = "<directory>";
+static const String	 kDirectoryExtStart  = "<directory+";
+static const String	 kCurrentDatePattern = "<currentdate>";
+static const String	 kCurrentTimePattern = "<currenttime>";
+static const String	 kTrackPattern	     = "<track>";
+static const String	 kTrackExtStart	     = "<track(";
+static const String	 kDiscPattern	     = "<disc>";
+static const String	 kDiscExtStart	     = "<disc(";
+static const String	 kAlbumArtistPattern = "<albumartist>";
+static const String	 kConductorPattern   = "<conductor>";
+static const String	 kComposerPattern    = "<composer>";
+static const String	 kTempoPattern	     = "<tempo>";
+static const String	 kInitialKeyPattern  = "<initialkey>";
+
 Void freac::Utilities::FillGenreList(List *list)
 {
 	BoCA::Config	*config = BoCA::Config::Get();
@@ -273,6 +298,21 @@ Void freac::Utilities::UpdateGenreList(List *list, const String &genre)
 }
 
 String freac::Utilities::GetOutputFileName(const BoCA::Config *config, const Track &track)
+{
+	String	 filePattern	 = config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderFilenamePatternID, Config::SettingsEncoderFilenamePatternDefault);
+	String	 outputDirectory = BoCA::Utilities::GetAbsolutePathName(config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderOutputDirectoryID, Config::SettingsEncoderOutputDirectoryDefault));
+	Bool	 writeToInputDir = config->GetIntValue(Config::CategorySettingsID, Config::SettingsWriteToInputDirectoryID, Config::SettingsWriteToInputDirectoryDefault);
+	String	 encoderID	 = config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderID, Config::SettingsEncoderDefault);
+	Bool	 useUnicode	 = config->GetIntValue(Config::CategorySettingsID, Config::SettingsFilenamesAllowUnicodeID, Config::SettingsFilenamesAllowUnicodeDefault);
+	Bool	 replaceSpaces	 = config->GetIntValue(Config::CategorySettingsID, Config::SettingsFilenamesReplaceSpacesID, Config::SettingsFilenamesReplaceSpacesDefault);
+	Bool	 useFallback	 = config->GetIntValue(Config::CategorySettingsID, Config::SettingsFilenamesUseFallbackID, Config::SettingsFilenamesUseFallbackDefault);
+
+	if (filePattern.Trim() == NIL) filePattern = Config::SettingsEncoderFilenamePatternDefault;
+
+	return GetOutputFileName(track, filePattern, outputDirectory, writeToInputDir, encoderID, True, useUnicode, replaceSpaces, useFallback);
+}
+
+String freac::Utilities::GetOutputFileName(const Track &track, const String &filePattern, const String &outputDirectory, Bool writeToInputDir, const String &encoderID, Bool normalizeFileName, Bool useUnicode, Bool replaceSpaces, Bool useFallback)
 {
 	if (track.outputFile != NIL) return track.outputFile;
 
@@ -282,9 +322,7 @@ String freac::Utilities::GetOutputFileName(const BoCA::Config *config, const Tra
 
 	/* Check if input file folder is writable.
 	 */
-	String	 shortInFileName = File(track.fileName).GetFileName().Head(File(track.fileName).GetFileName().FindLast("."));
-	String	 inFileDirectory = File(track.fileName).GetFilePath();
-	Bool	 writeToInputDir = config->GetIntValue(Config::CategorySettingsID, Config::SettingsWriteToInputDirectoryID, Config::SettingsWriteToInputDirectoryDefault);
+	String	 inFileDirectory = track.isCDTrack ? String() : File(track.fileName).GetFilePath();
 
 	if (writeToInputDir && (track.isCDTrack || !BoCA::Utilities::IsFolderWritable(inFileDirectory))) writeToInputDir = False;
 
@@ -293,7 +331,7 @@ String freac::Utilities::GetOutputFileName(const BoCA::Config *config, const Tra
 	String	 outputFileName;
 
 	if (writeToInputDir) outputFileName.Copy(inFileDirectory).Append(Directory::GetDirectoryDelimiter());
-	else		     outputFileName.Copy(BoCA::Utilities::GetAbsolutePathName(config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderOutputDirectoryID, Config::SettingsEncoderOutputDirectoryDefault)));
+	else		     outputFileName.Copy(outputDirectory);
 
 	/* Get file extension from selected encoder component
 	 * caching the result for requests in quick succession.
@@ -302,88 +340,121 @@ String freac::Utilities::GetOutputFileName(const BoCA::Config *config, const Tra
 	static String		 selectedEncoder = NIL;
 	static UnsignedInt64	 lastRequest	 = 0;
 
-	if (selectedEncoder != config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderID, Config::SettingsEncoderDefault) || S::System::System::Clock() - lastRequest >= 750)
+	if (selectedEncoder != encoderID || S::System::System::Clock() - lastRequest >= 750)
 	{
 		Registry		&boca	 = Registry::Get();
-		EncoderComponent	*encoder = (EncoderComponent *) boca.CreateComponentByID(config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderID, Config::SettingsEncoderDefault));
+		EncoderComponent	*encoder = (EncoderComponent *) boca.CreateComponentByID(encoderID);
 
 		fileExtension	= (encoder != NIL) ? encoder->GetOutputFileExtension() : "audio";
-		selectedEncoder	= config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderID, Config::SettingsEncoderDefault);
+		selectedEncoder	= encoderID;
 
 		boca.DeleteComponent(encoder);
 	}
 
 	lastRequest = S::System::System::Clock();
 
+	/* Check available metadata.
+	 */
+	Bool	 metadataAvailable = False;
+
+	if ((info.title	 != NIL &&  filePattern.Contains(kTitlePattern)						      ) ||
+	    (info.album	 != NIL &&  filePattern.Contains(kAlbumPattern)						      ) ||
+	    (info.artist != NIL && (filePattern.Contains(kArtistPattern) || filePattern.Contains(kAlbumArtistPattern))) ||
+	    (info.track	 >  0	&& (filePattern.Contains(kTrackPattern)	 || filePattern.Contains(kTrackExtStart)     ))) metadataAvailable = True;
+
 	/* Replace patterns.
 	 */
-	String	 filePattern = config->GetStringValue(Config::CategorySettingsID, Config::SettingsEncoderFilenamePatternID, Config::SettingsEncoderFilenamePatternDefault);
-
-	if (filePattern.Trim() == NIL) filePattern = Config::SettingsEncoderFilenamePatternDefault;
-
-	if (info.HasBasicInfo() || filePattern.Contains("<filename>") || filePattern.Contains("<currentdate>") || filePattern.Contains("<currenttime>"))
+	if (metadataAvailable || !useFallback || filePattern.Contains(kFileNamePattern) || filePattern.Contains(kCurrentDatePattern) || filePattern.Contains(kCurrentTimePattern))
 	{
-		String		 shortOutFileName = filePattern;
+		String	 shortOutFileName = filePattern;
 
-		DateTime	 currentDateTime  = DateTime::Current();
-		String		 currentDate	  = String().FillN('0', 3 - Math::Floor(Math::Log10(				      currentDateTime.GetYear()	     ))) .Append(String::FromInt(currentDateTime.GetYear()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(				      currentDateTime.GetMonth()     )))).Append(String::FromInt(currentDateTime.GetMonth()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(				      currentDateTime.GetDay()	     )))).Append(String::FromInt(currentDateTime.GetDay()));
-		String		 currentTime	  = String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetHour()	> 0 ? currentDateTime.GetHour()	  : 1))) .Append(String::FromInt(currentDateTime.GetHour()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMinute() > 0 ? currentDateTime.GetMinute() : 1)))).Append(String::FromInt(currentDateTime.GetMinute()));
-
-		shortOutFileName.Replace("<artist>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.artist.Length() > 0 ? info.artist : i18n->TranslateString("unknown artist")));
-		shortOutFileName.Replace("<title>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.title.Length() > 0 ? info.title : i18n->TranslateString("unknown title")));
-		shortOutFileName.Replace("<album>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.album.Length() > 0 ? info.album : i18n->TranslateString("unknown album")));
-		shortOutFileName.Replace("<genre>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.genre.Length() > 0 ? info.genre : i18n->TranslateString("unknown genre")));
-		shortOutFileName.Replace("<year>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.year > 0 ? String::FromInt(info.year) : i18n->TranslateString("unknown year")));
-		shortOutFileName.Replace("<discid>", CDDB::DiscIDToString(track.discid));
-		shortOutFileName.Replace("<filename>", BoCA::Utilities::ReplaceIncompatibleCharacters(shortInFileName));
-		shortOutFileName.Replace("<filetype>", fileExtension.ToUpper());
-		shortOutFileName.Replace("<currentdate>", currentDate);
-		shortOutFileName.Replace("<currenttime>", currentTime);
+		if (shortOutFileName.Contains(kArtistPattern))	 shortOutFileName.Replace(kArtistPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(info.artist.Length() > 0 ? info.artist : i18n->TranslateString("unknown artist")));
+		if (shortOutFileName.Contains(kTitlePattern))	 shortOutFileName.Replace(kTitlePattern, BoCA::Utilities::ReplaceIncompatibleCharacters(info.title.Length() > 0 ? info.title : i18n->TranslateString("unknown title")));
+		if (shortOutFileName.Contains(kAlbumPattern))	 shortOutFileName.Replace(kAlbumPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(info.album.Length() > 0 ? info.album : i18n->TranslateString("unknown album")));
+		if (shortOutFileName.Contains(kGenrePattern))	 shortOutFileName.Replace(kGenrePattern, BoCA::Utilities::ReplaceIncompatibleCharacters(info.genre.Length() > 0 ? info.genre : i18n->TranslateString("unknown genre")));
+		if (shortOutFileName.Contains(kYearPattern))	 shortOutFileName.Replace(kYearPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(info.year > 0 ? String::FromInt(info.year) : i18n->TranslateString("unknown year")));
+		if (shortOutFileName.Contains(kDiscIDPattern))	 shortOutFileName.Replace(kDiscIDPattern, CDDB::DiscIDToString(track.discid));
+		if (shortOutFileName.Contains(kFileTypePattern)) shortOutFileName.Replace(kFileTypePattern, fileExtension.ToUpper());
 
 		/* Replace <track> pattern.
 		 */
-		shortOutFileName.Replace("<track>", String(info.track < 10 ? "0" : NIL).Append(String::FromInt(info.track < 0 ? 0 : info.track)));
+		if (shortOutFileName.Contains(kTrackPattern)) shortOutFileName.Replace(kTrackPattern, String(info.track < 10 ? "0" : NIL).Append(String::FromInt(info.track < 0 ? 0 : info.track)));
 
-		for (Int i = 1; i <= 4; i++)
+		for (Int i = 1; i <= 4 && shortOutFileName.Contains(kTrackExtStart); i++)
 		{
-			String	 pattern = String("<track(").Append(String::FromInt(i)).Append(")>");
+			String	 pattern = String(kTrackExtStart).Append(String::FromInt(i)).Append(")>");
 
 			shortOutFileName.Replace(pattern, String().FillN('0', i - ((Int) Math::Log10(info.track > 0 ? info.track : 1) + 1)).Append(String::FromInt(info.track < 0 ? 0 : info.track)));
 		}
 
 		/* Replace <disc> pattern.
 		 */
-		shortOutFileName.Replace("<disc>", String::FromInt(info.disc < 0 ? 0 : info.disc));
+		if (shortOutFileName.Contains(kDiscPattern)) shortOutFileName.Replace(kDiscPattern, String::FromInt(Math::Max(1, info.disc)));
 
-		for (Int i = 1; i <= 4; i++)
+		for (Int i = 1; i <= 4 && shortOutFileName.Contains(kDiscExtStart); i++)
 		{
-			String	 pattern = String("<disc(").Append(String::FromInt(i)).Append(")>");
+			String	 pattern = String(kDiscExtStart).Append(String::FromInt(i)).Append(")>");
 
-			shortOutFileName.Replace(pattern, String().FillN('0', i - ((Int) Math::Log10(info.disc > 0 ? info.disc : 1) + 1)).Append(String::FromInt(info.disc < 0 ? 0 : info.disc)));
+			shortOutFileName.Replace(pattern, String().FillN('0', i - ((Int) Math::Log10(info.disc > 0 ? info.disc : 1) + 1)).Append(String::FromInt(Math::Max(1, info.disc))));
+		}
+
+		/* Replace <currentdate> pattern.
+		 */
+		if (shortOutFileName.Contains(kCurrentDatePattern))
+		{
+			DateTime	 currentDateTime = DateTime::Current();
+			String		 currentDate	 = String().FillN('0', 3 - Math::Floor(Math::Log10(currentDateTime.GetYear() ))) .Append(String::FromInt(currentDateTime.GetYear()))
+						   .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMonth())))).Append(String::FromInt(currentDateTime.GetMonth()))
+						   .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetDay()  )))).Append(String::FromInt(currentDateTime.GetDay()));
+
+			shortOutFileName.Replace(kCurrentDatePattern, currentDate);
+		}
+
+		/* Replace <currenttime> pattern.
+		 */
+		if (shortOutFileName.Contains(kCurrentTimePattern))
+		{
+			DateTime	 currentDateTime = DateTime::Current();
+			String		 currentTime	 = String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetHour()   > 0 ? currentDateTime.GetHour()	 : 1))) .Append(String::FromInt(currentDateTime.GetHour()))
+						   .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMinute() > 0 ? currentDateTime.GetMinute() : 1)))).Append(String::FromInt(currentDateTime.GetMinute()));
+
+			shortOutFileName.Replace(kCurrentTimePattern, currentTime);
 		}
 
 		/* Replace other text fields.
 		 */
 		foreach (const String &pair, info.other)
 		{
-			String	 key   = pair.Head(pair.Find(":"));
-			String	 value = pair.Tail(pair.Length() - pair.Find(":") - 1);
+			String	 key   = pair.Head(pair.Find(kColon));
+			String	 value = pair.Tail(pair.Length() - pair.Find(kColon) - 1);
 
 			if (value == NIL) continue;
 
-			if	(key == INFO_ALBUMARTIST) shortOutFileName.Replace("<albumartist>", BoCA::Utilities::ReplaceIncompatibleCharacters(value));
-			else if	(key == INFO_CONDUCTOR)	  shortOutFileName.Replace("<conductor>", BoCA::Utilities::ReplaceIncompatibleCharacters(value));
-			else if	(key == INFO_COMPOSER)	  shortOutFileName.Replace("<composer>", BoCA::Utilities::ReplaceIncompatibleCharacters(value));
+			if	(key == INFO_ALBUMARTIST && shortOutFileName.Contains(kAlbumArtistPattern)) shortOutFileName.Replace(kAlbumArtistPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(value));
+			else if	(key == INFO_CONDUCTOR	 && shortOutFileName.Contains(kConductorPattern))   shortOutFileName.Replace(kConductorPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(value));
+			else if	(key == INFO_COMPOSER	 && shortOutFileName.Contains(kComposerPattern))    shortOutFileName.Replace(kComposerPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(value));
+			else if	(key == INFO_BPM	 && shortOutFileName.Contains(kTempoPattern))	    shortOutFileName.Replace(kTempoPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(value));
+			else if	(key == INFO_INITIALKEY  && shortOutFileName.Contains(kInitialKeyPattern))  shortOutFileName.Replace(kInitialKeyPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(value));
 		}
 
-		if (info.artist.Length() > 0) shortOutFileName.Replace("<albumartist>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.artist));
+		if (info.artist.Length() > 0 && shortOutFileName.Contains(kAlbumArtistPattern)) shortOutFileName.Replace(kAlbumArtistPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(info.artist));
 
-		shortOutFileName.Replace("<albumartist>", BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown album artist")));
-		shortOutFileName.Replace("<conductor>", BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown conductor")));
-		shortOutFileName.Replace("<composer>", BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown composer")));
+		if (shortOutFileName.Contains(kAlbumArtistPattern)) shortOutFileName.Replace(kAlbumArtistPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown album artist")));
+		if (shortOutFileName.Contains(kConductorPattern))   shortOutFileName.Replace(kConductorPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown conductor")));
+		if (shortOutFileName.Contains(kComposerPattern))    shortOutFileName.Replace(kComposerPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown composer")));
+		if (shortOutFileName.Contains(kTempoPattern))	    shortOutFileName.Replace(kTempoPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown tempo")));
+		if (shortOutFileName.Contains(kInitialKeyPattern))  shortOutFileName.Replace(kInitialKeyPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown key")));
+
+		/* Replace <filename> pattern.
+		 */
+		if (shortOutFileName.Contains(kFileNamePattern))
+		{
+			String	 shortInFileName = File(track.fileName).GetFileName();
+
+			if (shortInFileName.Find(kDot) > 0) shortInFileName = shortInFileName.Head(shortInFileName.FindLast(kDot));
+
+			shortOutFileName.Replace(kFileNamePattern, BoCA::Utilities::ReplaceIncompatibleCharacters(shortInFileName));
+		}
 
 		/* Replace <directory> pattern.
 		 */
@@ -392,11 +463,11 @@ String freac::Utilities::GetOutputFileName(const BoCA::Config *config, const Tra
 		if	(directory[1] == ':')	       directory = directory.Tail(directory.Length() - 3);
 		else if (directory.StartsWith("\\\\")) directory = directory.Tail(directory.Length() - 2);
 
-		shortOutFileName.Replace("<directory>", directory);
+		if (shortOutFileName.Contains(kDirectoryPattern)) shortOutFileName.Replace(kDirectoryPattern, directory);
 
-		for (Int i = 0; i < 10 && shortOutFileName.Contains("<directory+"); i++)
+		for (Int i = 0; i < 10 && shortOutFileName.Contains(kDirectoryExtStart); i++)
 		{
-			String	 pattern = String("<directory+").Append(String::FromInt(i)).Append(">");
+			String	 pattern = String(kDirectoryExtStart).Append(String::FromInt(i)).Append(">");
 
 			if (shortOutFileName.Contains(pattern))
 			{
@@ -407,9 +478,9 @@ String freac::Utilities::GetOutputFileName(const BoCA::Config *config, const Tra
 				shortOutFileName.Replace(pattern, value);
 			}
 
-			for (Int j = 0; j < 10 && shortOutFileName.Contains(String("<directory+").Append(String::FromInt(i)).Append("(")); j++)
+			for (Int j = 0; j < 10 && shortOutFileName.Contains(String(kDirectoryExtStart).Append(String::FromInt(i)).Append("(")); j++)
 			{
-				String	 pattern = String("<directory+").Append(String::FromInt(i)).Append("(").Append(String::FromInt(j + 1)).Append(")>");
+				String	 pattern = String(kDirectoryExtStart).Append(String::FromInt(i)).Append("(").Append(String::FromInt(j + 1)).Append(")>");
 
 				if (!shortOutFileName.Contains(pattern)) continue;
 
@@ -435,11 +506,17 @@ String freac::Utilities::GetOutputFileName(const BoCA::Config *config, const Tra
 			}
 		}
 
-		Bool	 useUnicode    = config->GetIntValue(Config::CategorySettingsID, Config::SettingsFilenamesAllowUnicodeID, Config::SettingsFilenamesAllowUnicodeDefault);
-		Bool	 replaceSpaces = config->GetIntValue(Config::CategorySettingsID, Config::SettingsFilenamesReplaceSpacesID, Config::SettingsFilenamesReplaceSpacesDefault);
-
-		outputFileName.Append(BoCA::Utilities::ReplaceIncompatibleCharacters(shortOutFileName, useUnicode, False, replaceSpaces));
-		outputFileName = BoCA::Utilities::NormalizeFileName(outputFileName);
+		/* Replace incompatible characters and limit length of path components.
+		 */
+		if (normalizeFileName)
+		{
+			outputFileName.Append(BoCA::Utilities::ReplaceIncompatibleCharacters(shortOutFileName, useUnicode, False, replaceSpaces));
+			outputFileName = BoCA::Utilities::NormalizeFileName(outputFileName);
+		}
+		else
+		{
+			outputFileName.Append(shortOutFileName);
+		}
 	}
 	else if (track.isCDTrack)
 	{
@@ -451,12 +528,16 @@ String freac::Utilities::GetOutputFileName(const BoCA::Config *config, const Tra
 	}
 	else
 	{
+		String	 shortInFileName = File(track.fileName).GetFileName();
+
+		if (shortInFileName.Find(kDot) > 0) shortInFileName = shortInFileName.Head(shortInFileName.FindLast(kDot));
+
 		outputFileName.Append(shortInFileName);
 	}
 
 	/* Append file extension.
 	 */
-	if (fileExtension != NIL) outputFileName.Append(".").Append(fileExtension);
+	if (fileExtension != NIL) outputFileName.Append(kDot).Append(fileExtension);
 
 	return outputFileName;
 }
@@ -567,7 +648,7 @@ String freac::Utilities::GetSingleOutputFileNameDefault(const BoCA::Config *conf
 	BoCA::I18n	*i18n = BoCA::I18n::Get();
 
 	return BoCA::Utilities::NormalizeFileName(BoCA::Utilities::ReplaceIncompatibleCharacters(info.artist.Length() > 0 ? info.artist : i18n->TranslateString("unknown artist")).Append(" - ")
-					  .Append(BoCA::Utilities::ReplaceIncompatibleCharacters(info.album.Length()  > 0 ? info.album  : i18n->TranslateString("unknown album")))).Append(fileExtension != NIL ? "." : NIL).Append(fileExtension);
+					  .Append(BoCA::Utilities::ReplaceIncompatibleCharacters(info.album.Length()  > 0 ? info.album  : i18n->TranslateString("unknown album")))).Append(fileExtension != NIL ? kDot : String()).Append(fileExtension);
 }
 
 String freac::Utilities::GetPlaylistFileName(const BoCA::Config *config, const Track &track, const Array<Track> &originalTracks)
@@ -600,55 +681,73 @@ String freac::Utilities::GetPlaylistFileName(const BoCA::Config *config, const T
 
 	if (info.artist != NIL || info.album != NIL)
 	{
-		String		 shortOutFileName = config->GetStringValue(Config::CategoryPlaylistID, Config::PlaylistFilenamePatternID, Config::PlaylistFilenamePatternDefault);
+		String	 shortOutFileName = config->GetStringValue(Config::CategoryPlaylistID, Config::PlaylistFilenamePatternID, Config::PlaylistFilenamePatternDefault);
 
 		if (shortOutFileName.Trim() == NIL) shortOutFileName = Config::PlaylistFilenamePatternDefault;
 
-		DateTime	 currentDateTime  = DateTime::Current();
-		String		 currentDate	  = String().FillN('0', 3 - Math::Floor(Math::Log10(				      currentDateTime.GetYear()      ))) .Append(String::FromInt(currentDateTime.GetYear()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(				      currentDateTime.GetMonth()     )))).Append(String::FromInt(currentDateTime.GetMonth()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(				      currentDateTime.GetDay()	     )))).Append(String::FromInt(currentDateTime.GetDay()));
-		String		 currentTime	  = String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetHour()	> 0 ? currentDateTime.GetHour()	  : 1))) .Append(String::FromInt(currentDateTime.GetHour()))
-					    .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMinute()	> 0 ? currentDateTime.GetMinute() : 1)))).Append(String::FromInt(currentDateTime.GetMinute()));
-
-		shortOutFileName.Replace("<artist>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.artist.Length() > 0 ? info.artist : i18n->TranslateString("unknown artist")));
-		shortOutFileName.Replace("<album>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.album.Length() > 0 ? info.album : i18n->TranslateString("unknown album")));
-		shortOutFileName.Replace("<genre>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.genre.Length() > 0 ? info.genre : i18n->TranslateString("unknown genre")));
-		shortOutFileName.Replace("<year>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.year > 0 ? String::FromInt(info.year) : i18n->TranslateString("unknown year")));
-		shortOutFileName.Replace("<discid>", CDDB::DiscIDToString(track.discid));
-		shortOutFileName.Replace("<currentdate>", currentDate);
-		shortOutFileName.Replace("<currenttime>", currentTime);
+		if (shortOutFileName.Contains(kArtistPattern)) shortOutFileName.Replace(kArtistPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(info.artist.Length() > 0 ? info.artist : i18n->TranslateString("unknown artist")));
+		if (shortOutFileName.Contains(kAlbumPattern))  shortOutFileName.Replace(kAlbumPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(info.album.Length() > 0 ? info.album : i18n->TranslateString("unknown album")));
+		if (shortOutFileName.Contains(kGenrePattern))  shortOutFileName.Replace(kGenrePattern, BoCA::Utilities::ReplaceIncompatibleCharacters(info.genre.Length() > 0 ? info.genre : i18n->TranslateString("unknown genre")));
+		if (shortOutFileName.Contains(kYearPattern))   shortOutFileName.Replace(kYearPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(info.year > 0 ? String::FromInt(info.year) : i18n->TranslateString("unknown year")));
+		if (shortOutFileName.Contains(kDiscIDPattern)) shortOutFileName.Replace(kDiscIDPattern, CDDB::DiscIDToString(track.discid));
 
 		/* Replace <disc> pattern.
 		 */
-		shortOutFileName.Replace("<disc>", String::FromInt(info.disc < 0 ? 0 : info.disc));
+		if (shortOutFileName.Contains(kDiscPattern)) shortOutFileName.Replace(kDiscPattern, String::FromInt(Math::Max(1, info.disc)));
 
-		for (Int i = 1; i <= 4; i++)
+		for (Int i = 1; i <= 4 && shortOutFileName.Contains(kDiscExtStart); i++)
 		{
-			String	 pattern = String("<disc(").Append(String::FromInt(i)).Append(")>");
+			String	 pattern = String(kDiscExtStart).Append(String::FromInt(i)).Append(")>");
 
-			shortOutFileName.Replace(pattern, String().FillN('0', i - ((Int) Math::Log10(info.disc > 0 ? info.disc : 1) + 1)).Append(String::FromInt(info.disc < 0 ? 0 : info.disc)));
+			shortOutFileName.Replace(pattern, String().FillN('0', i - ((Int) Math::Log10(info.disc > 0 ? info.disc : 1) + 1)).Append(String::FromInt(Math::Max(1, info.disc))));
+		}
+
+		/* Replace <currentdate> pattern.
+		 */
+		if (shortOutFileName.Contains(kCurrentDatePattern))
+		{
+			DateTime	 currentDateTime = DateTime::Current();
+			String		 currentDate	 = String().FillN('0', 3 - Math::Floor(Math::Log10(currentDateTime.GetYear() ))) .Append(String::FromInt(currentDateTime.GetYear()))
+						   .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMonth())))).Append(String::FromInt(currentDateTime.GetMonth()))
+						   .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetDay()  )))).Append(String::FromInt(currentDateTime.GetDay()));
+
+			shortOutFileName.Replace(kCurrentDatePattern, currentDate);
+		}
+
+		/* Replace <currenttime> pattern.
+		 */
+		if (shortOutFileName.Contains(kCurrentTimePattern))
+		{
+			DateTime	 currentDateTime = DateTime::Current();
+			String		 currentTime	 = String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetHour()   > 0 ? currentDateTime.GetHour()	 : 1))) .Append(String::FromInt(currentDateTime.GetHour()))
+						   .Append(String().FillN('0', 1 - Math::Floor(Math::Log10(currentDateTime.GetMinute() > 0 ? currentDateTime.GetMinute() : 1)))).Append(String::FromInt(currentDateTime.GetMinute()));
+
+			shortOutFileName.Replace(kCurrentTimePattern, currentTime);
 		}
 
 		/* Replace other text fields.
 		 */
 		foreach (const String &pair, info.other)
 		{
-			String	 key   = pair.Head(pair.Find(":"));
-			String	 value = pair.Tail(pair.Length() - pair.Find(":") - 1);
+			String	 key   = pair.Head(pair.Find(kColon));
+			String	 value = pair.Tail(pair.Length() - pair.Find(kColon) - 1);
 
 			if (value == NIL) continue;
 
-			if	(key == INFO_ALBUMARTIST) shortOutFileName.Replace("<albumartist>", BoCA::Utilities::ReplaceIncompatibleCharacters(value));
-			else if	(key == INFO_CONDUCTOR)	  shortOutFileName.Replace("<conductor>", BoCA::Utilities::ReplaceIncompatibleCharacters(value));
-			else if	(key == INFO_COMPOSER)	  shortOutFileName.Replace("<composer>", BoCA::Utilities::ReplaceIncompatibleCharacters(value));
+			if	(key == INFO_ALBUMARTIST && shortOutFileName.Contains(kAlbumArtistPattern)) shortOutFileName.Replace(kAlbumArtistPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(value));
+			else if	(key == INFO_CONDUCTOR	 && shortOutFileName.Contains(kConductorPattern))   shortOutFileName.Replace(kConductorPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(value));
+			else if	(key == INFO_COMPOSER	 && shortOutFileName.Contains(kComposerPattern))    shortOutFileName.Replace(kComposerPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(value));
+			else if	(key == INFO_BPM	 && shortOutFileName.Contains(kTempoPattern))	    shortOutFileName.Replace(kTempoPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(value));
+			else if	(key == INFO_INITIALKEY	 && shortOutFileName.Contains(kInitialKeyPattern))  shortOutFileName.Replace(kInitialKeyPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(value));
 		}
 
-		if (info.artist.Length() > 0) shortOutFileName.Replace("<albumartist>", BoCA::Utilities::ReplaceIncompatibleCharacters(info.artist));
+		if (info.artist.Length() > 0 && shortOutFileName.Contains(kAlbumArtistPattern)) shortOutFileName.Replace(kAlbumArtistPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(info.artist));
 
-		shortOutFileName.Replace("<albumartist>", BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown album artist")));
-		shortOutFileName.Replace("<conductor>", BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown conductor")));
-		shortOutFileName.Replace("<composer>", BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown composer")));
+		if (shortOutFileName.Contains(kAlbumArtistPattern)) shortOutFileName.Replace(kAlbumArtistPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown album artist")));
+		if (shortOutFileName.Contains(kConductorPattern))   shortOutFileName.Replace(kConductorPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown conductor")));
+		if (shortOutFileName.Contains(kComposerPattern))    shortOutFileName.Replace(kComposerPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown composer")));
+		if (shortOutFileName.Contains(kTempoPattern))	    shortOutFileName.Replace(kTempoPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown tempo")));
+		if (shortOutFileName.Contains(kInitialKeyPattern))  shortOutFileName.Replace(kInitialKeyPattern, BoCA::Utilities::ReplaceIncompatibleCharacters(i18n->TranslateString("unknown key")));
 
 		playlistFileName.Append(BoCA::Utilities::ReplaceIncompatibleCharacters(shortOutFileName, useUnicode, False, replaceSpaces));
 	}
